library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library std; -- for Printing
use std.textio.all;
use ieee.std_logic_textio.all;

use work.core_pack.all;
use work.op_pack.all;

entity alu_tb is
end entity;

architecture beh of alu_tb is

	component alu is 
		port(
		op   : in  alu_op_type;
		A, B : in  std_logic_vector(DATA_WIDTH-1 downto 0);
		R    : out std_logic_vector(DATA_WIDTH-1 downto 0);
		Z    : out std_logic;
		V    : out std_logic);		
	end component;
	
	constant CLK_PERIOD : time := 10 ns;

	signal op : alu_op_type := ALU_NOP;
	signal A, B : std_logic_vector(DATA_WIDTH-1 downto 0) := x"00000000";
	signal R : std_logic_vector(DATA_WIDTH-1 downto 0);
	signal Z : std_logic := '0';
	signal V : std_logic := '0';
	
	file input_file : text;
	file output_ref_file : text;

begin
	-----------------------------------------------
	--   Instantiate your top level entity here  --
	-----------------------------------------------
	-- The testbench generates the above singals --
	-- use them as inputs to your system         --
	-----------------------------------------------
	uut : alu 
	port map (
		op => op,
		A => A,
		B => B,
		R => R,
		Z => Z,
		V => V
	);

	-- Generates the clock signal
		
	input : process
	
	-- file io related variables
		variable fstatus: file_open_status;
		variable l : line;
		
		variable A_temp : std_logic_vector(DATA_WIDTH-1 downto 0);
		variable B_temp : std_logic_vector(DATA_WIDTH-1 downto 0);
		variable op_temp : alu_op_type;
		
		variable i : integer range 0 to 2 := 0;
		
		procedure timeout(num_cycles : integer) is
		begin
			wait for CLK_PERIOD*num_cycles;
		end procedure;
			
	begin
	
		file_open(fstatus, input_file,"testdata/alu_input.txt", READ_MODE);
		
		while not endfile(input_file) loop
			readline(input_file, l); 
			if (l'length>=1) and l(1) = '#' then
				next;
			elsif (l'length<1) then
				next;
			end if; 
			
			case i is
				when 0 =>
					if l'length = 6 and l(1) /= '#' then
						case l(1 to 6) is 
							when "ALU_OR" =>
								op_temp := ALU_OR;
							when "alu_or" =>
								op_temp := ALU_OR;
							when others =>
								assert false report "No such alu operation" severity error;
						end case;
					end if;
					
					if l'length = 7 and l(1) /= '#' then
						case l(1 to 7) is 
							when "ALU_NOP" =>
								op_temp := ALU_NOP;
							when "ALU_SLT" =>
								op_temp := ALU_SLT;
							when "ALU_SLL" =>
								op_temp := ALU_SLL;
							when "ALU_SRL" =>
								op_temp := ALU_SRL;
							when "ALU_SRA" =>
								op_temp := ALU_SRA;
							when "ALU_ADD" =>
								op_temp := ALU_ADD;
							when "ALU_SUB" =>
								op_temp := ALU_SUB;
							when "ALU_AND" =>
								op_temp := ALU_AND;
							when "ALU_XOR" =>
								op_temp := ALU_XOR;
							when "ALU_NOR" =>
								op_temp := ALU_NOR;
							when "ALU_LUI" =>
								op_temp := ALU_LUI;
							
							when "alu_nop" =>
								op_temp := ALU_NOP;
							when "alu_slt" =>
								op_temp := ALU_SLT;
							when "alu_sll" =>
								op_temp := ALU_SLL;
							when "alu_srl" =>
								op_temp := ALU_SRL;
							when "alu_sra" =>
								op_temp := ALU_SRA;
							when "alu_add" =>
								op_temp := ALU_ADD;
							when "alu_sub" =>
								op_temp := ALU_SUB;
							when "alu_and" =>
								op_temp := ALU_AND;
							when "alu_xor" =>
								op_temp := ALU_XOR;
							when "alu_nor" =>
								op_temp := ALU_NOR;
							when "alu_lui" =>
								op_temp := ALU_LUI;
							when others =>
								assert false report "No such alu operation" severity error;
						end case;
					end if;
					
					if l'length = 8 and l(1) /= '#' then
						case l(1 to 8) is 
							when "ALU_SLTU" =>
								op_temp := ALU_SLTU;
							when "alu_sltu" =>
								op_temp := ALU_SLTU;
							when others =>
								assert false report "No such alu operation" severity error;
						end case;
					end if;
					
					op <= op_temp;
				when 1 =>
					read (l, A_temp);
					A <= A_temp;
				when 2 =>
					read (l, B_temp);
					B <= B_temp;
					timeout(2);
			end case;
				
			if i < 2 then
				i := i + 1;
			else 
				i := 0;
			end if;
			
		end loop;
	
		wait;	
	end process input;
	
	
	check_output : process
		-- file io related variables
		variable fstatus: file_open_status;
		variable l : line;
		
		variable R_temp : std_logic_vector(DATA_WIDTH-1 downto 0);
		variable V_temp : std_logic;
		variable Z_temp : std_logic;
		
		variable i : integer range 0 to 2 := 0;
		
		function slv_to_string(slv : std_logic_vector) return string is
		variable s : string(1 to DATA_WIDTH);
		begin
			for i in 0 to DATA_WIDTH-1 loop
				case slv(DATA_WIDTH-1 - i) is
					when '0' =>
						s(i+1) := '0';
					when '1' =>
						s(i+1) := '1';
					when 'X' =>
						s(i+1) := 'X';
					when 'U' =>
						s(i+1) := 'U';
					when 'H' =>
						s(i+1) := 'H';
					when 'Z' =>
						s(i+1) := 'Z';
					when 'L' =>
						s(i+1) := 'L';
					when 'W' =>
						s(i+1) := 'W';
					when '-' =>
						s(i+1) := '-';
					when others =>
						s(i+1) := 'E';
				end case;
			end loop;
			return s;
		end function;
		
		function sl_to_string(lv : std_logic) return string is
		begin
			case lv is
				when '0' =>
					return "0";
				when '1' =>
					return "1";
				when 'X' =>
					return "X";
				when 'U' =>
					return "U";
				when 'H' =>
					return "H";
				when 'Z' =>
					return "Z";
				when 'L' =>
					return "L";
				when 'W' =>
					return "W";
				when '-' =>
					return "-";
				when others =>
					return "E";
			end case;
		end function;
		
		function op_to_string(alu_op : alu_op_type) return string is
		begin
			case alu_op is
				when ALU_NOP =>
					return "ALU_NOP";
				when ALU_SLT =>
					return "ALU_SLT";
				when ALU_SLL =>
					return "ALU_SLL";
				when ALU_SRL =>
					return "ALU_SRL";
				when ALU_SRA	=>
					return "ALU_SRA";
				when ALU_ADD =>
					return "ALU_ADD";
				when ALU_SUB	=>
					return "ALU_SUB";
				when ALU_AND =>
					return "ALU_AND";
				when ALU_XOR =>
					return "ALU_XOR";
				when ALU_NOR =>
					return "ALU_NOR";
				when ALU_LUI =>
					return "ALU_LUI";
				when ALU_OR =>
					return "ALU_OR";
				when ALU_SLTU =>
					return "ALU_SLTU";
				when others =>
					assert false report "Wrong alu operation" severity error;
					return "Wrong alu operation";
			end case;
		end function;
		
		procedure timeout(num_cycles : integer) is
		begin
			wait for CLK_PERIOD*num_cycles;
		end procedure;
	begin
		file_open(fstatus, output_ref_file,"testdata/alu_output_ref.txt", READ_MODE);
		
		while not endfile(output_ref_file) loop
			readline(output_ref_file, l);
			if (l'length>=1) and l(1) = '#' then
				next;
			elsif (l'length<1) then
				next;
			end if; 
			
			case i is
				when 0 =>
					read (l, R_temp);
				when 1 =>
					read (l, Z_temp);
				when 2 =>
					read (l, V_temp);
					timeout(2);
					assert (R = R_temp and V = V_temp and Z = Z_temp) report "Failed test: op=" & op_to_string(op) & " A=" & slv_to_string(A) & " B=" & slv_to_string(B) & 
					", result: R=" & slv_to_string(R) & " Z=" & sl_to_string(Z) & " V=" & sl_to_string(V) & 
					", expected R=" & slv_to_string(R_temp) & " Z=" & sl_to_string(Z_temp) & " V=" & sl_to_string(V_temp) severity error;
			end case;	
			
			if i < 2 then
				i := i + 1;
			else 
				i := 0;
			end if;
			
		end loop;
		wait;
	end process;
	
end architecture;
